********************************
*
; BouQLder QLash
; check and handle amoeba
; (C) W. Lenerz 2018
*
********************************

; algo:
; increase nbr of amoeba found
; check whether a previous amoeba could grow, if yes just leave
; check whther this amoeba can grow, if not just leave
; set flag showing that this amoeba could grow
; try to grow

	section action

	include 'dev1_basic_boulder_asm_keys'
	include dev8_keys_sbasic

	xdef	amob_act
	xdef	rnd

	xref	drw_spr
	xref	drw_blk
	xref	exp_set


; d0   r   0 if OK, 1 if diggeridoo is dead, -1 if dying
; d2 c s   the object type
; d4 c	p  sprite animation counter
; d5 c	p  loop counter (row|col)
; d7 c	p  nbr of ticks betwen full scans
; a0 c	p  channel ID
; a3 c	p  timer address
; a4 c	p  ptr to NEXT element in array
; a5 c	p  workspace
; a6 c	p  basic ptr

; all other regs are volatile and may be used at will

; check amoeba during scan :

amob_act
	addq.w	#1,amo_nbr(a5)		; increase nbr of amoebaes
	tst.w	amo_gro(a5)		; can a previous amoeba grow?
	beq.s	amo_out 		; ... yes no further check necessary
	move.w	-4(a4),d0		; ... not yet, check to the left
	bsr.s	chk_gro 		; dosn't come back here if can grow
	move.w	(a4),d0 		; ... not yet, check to the right
	bsr.s	chk_gro 		; dosn't come back here if can grow
	move.w	-rowsiz2(a4),d0 	; ... not yet, check above
	bsr.s	chk_gro 		; dosn't come back here if can grow
	move.w	rowsize-2(a4),d0	; ... not yet, check below
	bsr.s	chk_gro 		; dosn't come back here if can grow
	bra.s	amo_out 		; this ameaba couldn't grow

; check whether amob can grow, DOES NOT RETURN TO CALLER IF IT CAN GROW
;			      ==========================================
chk_gro subq.w	#dirt,d0
	beq.s	can_gro
	subq.w	#space-dirt,d0
	beq.s	can_gro
	rts

is_gro	clr.w	amo_gro(a5)		; show growth is possible
amo_out moveq	#0,d0
	rts

; this amoeba may grow - but it does so randomly  CHANGES A7
; get a random nbr which indicates the chances whether this amoeba will grow
; if it does grow, use this as the direction in which this amoeba wants to grow
; check whether growth in this direction is possible, if yes, grow there
can_gro
	addq.l	#4,a7			; get rid of return address
	move.l	#$800001,d2		; range from 1 to 128 - slow growth
	tst.w	amo_kwik(a5)		; or are we growing quickly?
	bpl.s	get_rnd 		; ... no
	move.l	#$100001,d2		; ... yes; so this is the random range
get_rnd bsr.s	rnd			, get randow nbr
	cmp.w	#4,d1			; 4 chances for growth
	bgt.s	is_gro			; amo didn't grow this time....
	mulu	#2,d1			; word access (faster than lsl on q68)
	move.w	dir_tble(pc,d1.w),a1	; offset from A4 to get to square
	add.l	a4,a1
	move.w	(a1),d0 		; square to try to expand into
	subq.w	#dirt,d0		; is it dirt?
	bne.s	no_drt			; ... no, could be space
	move.w	#amoeba,(a1)		; ... yes; it's now an amoeba
	addq.w	#1,amo_nbr(a5)		; so there's one more of 'em
	bra.s	is_gro			; done
no_drt	subq.w	#4,d0			; is it space ?
	bne.s	is_gro			; ... no, so I can't expand into that square
	move.w	#amoeba,(a1)		; ... yes; it's now an amoeba
	addq.w	#1,amo_nbr(a5)		; so there's one more of 'em
	bra.s	is_gro

; get a random number (inspired by smsqe code)
; d0	s
; d1   r    random nbr in .w
; D2 c	s   top | bottom of range
; d3	s
rnd	move.l	sb_rand(a6),d0
	move.w	d0,d3
	swap	d0
	mulu	#$c12d,d0		; multiply up
	mulu	#$712d,d3
	swap	d0
	clr.w	d0
	add.l	d0,d3
	addq.l	#1,d3
	move.l	d3,sb_rand(a6)		; and save it

	move.w	d2,d1			; bottom of range
	swap	d2			; top of range

	addq.w	#1,d2
	sub.w	d1,d2			; size of range

	swap	d3			; use msword
	mulu	d2,d3			;
	swap	d3			; and overflow
	add.w	d3,d1			; add to base

	rts
; my position is at -2(a4). These are the offsets to add to a4 get get to the
; squares left, right, above or below mine, from my current position
dir_tble
	dc.w	0			; bogus
	dc.w	-4			; left
	dc.w	0			; right
	dc.w	-rowsiz2		; above
	dc.w	rowsize-2		; below



	end
