****************************************************
*
; BouQLder QLash
; Scan the playing field
; (C) W. Lenerz 2018
*
*****************************************************
     
; This contains the main scan loops.

; A timer is decreased by one at every frame interrupt, giving a basic cycle of
; 50 or 60 Hz. One cycle = 1 tick.

; A calibration is made by the basic program before starting the game, to see
; whether we'r running on a 50 or 60 Hz machine. Then either 5 or 6 is stored
; at delay(a5) so that delay(a5) ticks = 1/10th of a second.


; This then uses 2 scan loops:


; 1 - every 1/10th of a second a "full" scan is made to update the game,  i.e
;   check diggeridoo movement
;   check & move falling boulders/diamonds
;   move flies
;   check for death
;   check for diamond collection etc

					
; 2 - Every second tick between every full scan - time permitting - the animated
;   sprites are animated. So, if a full scan took a long time, the sprite
;   animation may occur less frequently.


; the caves are made up of a 22 rows * 40 cols grid =880 squares
; the array is DIMmed as DIM cv_data%(22,41)
; cols 0 and 41 are always 0
; cols 1 and 40 are always steel wall (except for intermissions)
; row  0 is always 0
; rows 1 and 22 are always steel wall


	section basic

	include 'dev1_basic_boulder_asm_snd_keys'
	include 'dev1_basic_boulder_asm_keys'
	include 'dev8_keys_qlv'
	include 'dev8_keys_err'
	include 'dev8_keys_sbasic'
	include 'dev8_keys_qdos_io'
	include 'DEV8_keys_qdos_sms'

	xdef	bd_scan
	xdef	drw_spr 		; draw a sprite
	xdef	drw_spr2
	xdef	drw_blk 		; draw a black block
	xdef	drw_blk2		; draw a black block w/o calc position
	xdef	rt_int

	xref	mv_diggr		; check diggeridoo movement
	xref	drw_time		; draw time, in status_asm
	xref	sound
	xref	rnd			; in amoeba_asm
	xref	drw_score		; in status_asm
	xref	drw_time		; "   "
	xref	exp_set 		; in explode_asm
					

dead	bsr	drw_time		; called when you're dead and time expired
	moveq	#-1,d4
	bra.s	rt_int2
leave	move.w	d1,d4

; rt_int:  returns an integer to basic
; on entry, d4 contains integer to return
; the returns are :
; -1 = you're dead
;  0 = ESC  hit
;  1 = ENTER (go to next level, debug version only)
;  2 = finished this cave
; 65 = space+ enter (go back one level, debug version only)
rt_int2 move.l	chanID(a5),a0		; first empty the keyboard buffer
	moveq	#1,d3
rt_lp	lea	buffer(a5),a1
	moveq	#bufsiz,d2
	moveq	#iob.fmul,d0
	trap	#3
	tst.l	d0
	bne.s	rt_int
	sub.w	d2,d1			; did we get as many bytes as possible?
	beq.s	rt_lp			; yes, so there might be more

rt_int	move.l	oscore(a5),points(a5)	; set permanent score
	moveq	#6,d1
	move.w	qa.resri,a2
	move.l	sb_arthp(a6),a1
	jsr	(a2)			; get enough spce on ari stack
	subq.l	#2,sb_arthp(a6)
	move.l	sb_arthp(a6),a1
	move.w	d4,(a6,a1.l)
	moveq	#3,d4
	moveq	#0,d0
	rts

errbp	moveq	#err.ipar,d0
	rts

;******************************
;
; BD_SCAN wark_area
;
*******************************

bd_scan move.w	sb.gtlin,a2
	jsr	(a2)			; get long integer
	bne.s	errbp
	subq.w	#1,d3			; there must be only 1
	bne.s	errbp			; but there wasn't
	addq.l	#4,sb_arthp(a6) 	; adjust maths stack
	move.l	(a6,a1.l),a5		; pointer to work space
	move.l	timer(a5),a3
	move.l	scr_off(a5),a0		; address 1st row in screen
	move.w	#10,onesec(a5)		; if this becomes 0, 1 sec has gone by
	clr.w	mw_on(a5)		; magic wall is off
	moveq	#0,d7
	move.w	delay(a5),d7		; ticks to next full scan

; d7 = nbr of ticks between full scans
; a0 = screen 1st displayable row/col
; a3 = timer
; a5 = workspace
; a6 = basic ptr
big_loop
	move.l	d7,d2			; ticks for a full scan
	move.w	d7,(a3) 		; reset timer if (a3)=0, 1/10th of a second has gone by
	move.l	a0,-(a7)
	moveq	#-1,d1
	moveq	#1,d3
	sub.l	a1,a1
	moveq	#8,d0
	trap	#1			; sleep a little
	move.l	(a7)+,a0
tmr_loop
	move.w	(a3),d0 		; timer ticks
	ble.s	do_scan 		; 1/10th of a sec expired, do full scan now
	cmp.w	d0,d2			; still the same tick?
	beq.s	tmr_loop		; yes, wait at least a tick
	move.w	d0,-(a7)		; keep tick
	bsr	animate 		; animate sprites
	move.w	(a7)+,d2		; get old tick back
	bra.s	tmr_loop

; get a key via keyrow,
; on return d1.b = what :
; 1  enter
; 2  left
; 4  up
; 8  esc
; 16 right
; 64 space
; 66 space + left
; 68 space + up
; 80 space + right
;128 down
;192 space + down
gk_regs reg d5/d7/a0/a3
getkey	movem.l gk_regs,-(a7)
	lea	keybfr(a5),a3		; keyrow parameter block
	moveq	#sms.hdop,d0
	move.l	chanID(a5),a0
	trap	#do.sms2
	movem.l (a7)+,gk_regs
	and.w	#$ff,d1
	rts

*********************************************************************
*
* scan the entire playing field once for a full scan
*
*********************************************************************
do_scan
	move.l	array(a5),a4		; start at beginning of array
	subq.l	#8,a4			; see label jmpelmt below
	clr.w	amo_nbr(a5)		; pretend there are no amoebae
	moveq	#nbrrows-2,d5		; there are 20 rows to inspect
	moveq	#0,d4			; no animated sprites here
	subq.w	#1,onesec(a5)		; 10 of these ticks should make a second
	bne.s	nosec			; a second hasn't gone by yet
; update display if a second has gone by, reduce times remaining
	move.w	#10,onesec(a5)		; new ticks number
	bchg	#1,exitbit(a5)		; this is now 0 or 2
	move.b	entry(a5),d0		; is diggeridoo still entering the scene?
	beq.s	cont1			; ... no
	subq.b	#1,entry(a5)		; ... yes, not long now before he's here
	bne	sca_row 		; ... but not yet
	move.l	rok_pos(a5),a1		; diggeridoo is here (position in array)
	clr.w	(a1)			; he'll be shown here now
	move.l	#snd_exit,d1
	bsr	sound			; make arrival sound

cont1	subq.w	#1,amo_kwik(a5) 	; 1 second less until amoeba grow fast
	tst.w	mw_on(a5)		; is magic wall on?
	ble.s	no_mwl			; ... no
	subq.w	#1,mw_on(a5)		; ... yes, 1 second less magic wall time
no_mwl	subq.w	#1,time(a5)		; time remaining to finish cave
	bgt.s	alive			; there still is some
; when no more time is left, diggeridoo explodes and then you leave this cave
; when diggeridoo dies otherwise, you have yo hit esc to leave the cave, else
; you just stay there until the time runs out.
	tst.w	dying(a5)
	bgt	dead			; time ran out and we're already dead
	bne.s	rf_dyin 		; we're already exploding
	move.l	rok_pos(a5),a1
	moveq	#-1,d2
	move.w	#explosion,d2		; will explode to space
	bsr	exp_set 		; start explosion
	move.w	#-1,dying(a5)		; first stage of explosion is now on

alive	bsr	drw_time		; show new time

nosec	tst.b	entry(a5)		; diggeridoo still entering?
	beq.s	cont2			; ... no
	bchg	#1,exitbit(a5)		; ...this is now 0 or 2
	bra.s	sca_row 		; (redraws enter portal)

cont2	st	amo_gro(a5)		; pretend amos can't grow anymore
exp_sta bchg	#1,explo(a5)		; set explosion delay status
gkey	bsr	getkey			; get key into D1

tstdead tst.w	dying(a5)		; diggeridoo status
	beq.s	digg_ok 		; ... diggeridoo is ok
dig_rip blt.s	rf_dyin 		; ... diggeridoo is dying
	moveq	#-1,d4			; ... diggeridoo is dead preset return value
	cmp.b	#8,d1			; esc presed?
	beq	rt_int2 		; ... yes, return this to basic
	tst.w	time(a5)		; time ran out?
	beq	rt_int2 		; ... yes, done
	bra	big_loop		; ... no, wait till it does

rf_dyin subq.w	#1,dying(a5)		; diggeridoo is still dying (exploding)
	cmp.w	#-11,dying(a5)		; is explosion over
	bne.s	sca_row 		; ... not yet
	move.w	#1,dying(a5)		; ... yes, signal dead status
	bra.s	sca_row 		; redraw

digg_ok 				; diggeridoo is neither dead nor dying
	move.w	d1,d0			; key hit
;;;;;	     andi.b  #9,d0		     ; ESC/Enter hit? DEBUG
	andi.b	#8,d0			; ESC/Enter hit?
	bne	leave			; ... yes, leave
cont	bsr	mv_diggr		; possibly move diggeridoo
	bne	won_cve 		; we finished this cave

; d5 = counter (row | col)
; d7 = loop delay
; a0 = screen (start of game content at steel walls)
; a3 = ptr to timer
; a4 = start of row 2 of array (absolute)
; a5 = workspace
; a6 = basic ptr

sca_row swap	d5
	move.w	#nbrcols-2,d5		; there are 38 cols to check per row
jmpelmt addq.w	#8,a4			; jump over 0th & 1st elmnt of this row
					; and last 2 elmnts at end of previous row
sca_col move.w	(a4)+,d0		; get type
	blt.s	reverse 		; negative means skip this, but reset
	move.w	d0,d2			; keep for action routines
	add.w	d0,d0
	move.w	jmp_tbl(pc,d0.w),d0	; ptr to action routine
	ble.s	do_col			; do nothing
	jsr	jmp_tbl(pc,d0.w)	; all the action routines have the above
					; regs (A4 = NEXT element in array)
					; with D2 = current element. They must
					; return a status in D0: 0 = OK,
					; 1 = diggeridoo is dead/dying
	tst.w	dying(a5)		; am I currently dying?
	bne.s	scan_go 		; ... yes, no need to set status
	move.w	d0,dying(a5)		;
	beq.s	scan_go
	move.w	#12,time(a5)
scan_go moveq	#0,d0
do_col	dbf	d5,sca_col		; treat all cols
sca_lp	swap	d5
	dbf	d5,sca_row		; treat all rows
	cmp.w	#max_amo,amo_nbr(a5)	; too many amoebae ?
	ble.s	tst_gro 		; ... no
	move.w	#1,amo_chg(a5)		; ... yes, turn them to boulders
	bra.s	tst_spc
tst_gro tst.w	amo_gro(a5)		; amoeba can still grow?
	beq.s	tst_spc 		; ... yes
	move.w	#-1,amo_chg(a5) 	; ... no, change to dias in next scan loop

; check for special space drawing
tst_spc tst.b	black(a5)		; was special space drawing?
	beq.s	tst_pan 		; ... no, next loop
	move.w	#$00ff,black(a5)	; ... yes, draw black
tst_pan bclr	#0,panned(a5)		; panned ?
	beq	big_loop		; ... no, animation is done
	tst.b	fast(a5)		; SMSQmulator, QPC?
	beq	big_loop		; ... yes, wait a bit as usual
	move.l	d7,d2			; no, go faster
	subq.w	#2,d2
	move.w	d2,(a3)
	bra	tmr_loop

; a sprite shouldn't be acted upon during this scan, reset it now
reverse neg.w	d0
	move.w	d0,-2(a4)
	bra.s	do_col

; table of action routines during full scan : all the action routines should
; return a status in D0: 0 = OK, 1 = diggeridoo is dead/dying
	xref	ffly_act
	xref	bfly_act
	xref	bldrfal
	xref	new_spc
	xref	explode
	xref	amob_act
	xref	diafall

no_actn equ	0
jmp_tbl dc.w	no_actn 		; diggeridoo
	dc.w	no_actn 		; boulder
	dc.w	no_actn 		; spare
	dc.w	no_actn 		; dirt
	dc.w	no_actn 		; steel wall
	dc.w	no_actn 		; brick wall
	dc.w	no_actn 		; expanding wall
	dc.w	no_actn 		; space
	dc.w	no_actn 		; magic wall
	dc.w	no_actn 		; diamond
	dc.w	amob_act-jmp_tbl	; amoeba	10

	dc.w	ffly_act-jmp_tbl	; ffly_act
	dc.w	ffly_act-jmp_tbl	; ffly_act
	dc.w	ffly_act-jmp_tbl	; ffly_act
	dc.w	ffly_act-jmp_tbl	; ffly_act

	dc.w	bfly_act-jmp_tbl	; butterfly	15
	dc.w	bfly_act-jmp_tbl	; butterfly
	dc.w	bfly_act-jmp_tbl	; butterfly
	dc.w	bfly_act-jmp_tbl	; butterfly

	dc.w	explode-jmp_tbl 	; explosion	19 explode to space
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion	23
	
	dc.w	explode-jmp_tbl 	; explosion	24 explode to diamond
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion
	dc.w	explode-jmp_tbl 	; explosion	28
	
	dc.w	no_actn 		;		29 spare
	dc.w	no_actn 		; boulder rolling left
	dc.w	no_actn 		;	  rolling right
	dc.w	bldrfal-jmp_tbl 	;	  falling
	dc.w	new_spc-jmp_tbl 	; newly created space 33
	dc.w	no_actn 		; diamond rolling left
	dc.w	no_actn 		;	  rolling right
	dc.w	diafall-jmp_tbl 	;	  falling
	dc.w	no_actn 		;		 33
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;		 40
	dc.w	no_actn 		;
	dc.w	no_actn 		;
	dc.w	no_actn 		;		 43


; we finished the cave, get bonus points and leave
won_cve move.w	time(a5),d6
	bra.s	do_lp			; nbr of secs remaining = nbr of bonus pts
done_lp addq.l	#1,points(a5)		; increase score
	move.l	#snd_inc,d1
	bsr	sound			; make appropriate sound
	bsr	drw_score		; show new score
	subq.w	#1,time(a5)
	bsr	drw_time
	move.l	a0,-(a7)
	moveq	#-1,d1
	moveq	#2,d3
	sub.l	a1,a1
	moveq	#8,d0
	trap	#1			; sleep a little
	move.l	(a7)+,a0
do_lp	dbf	d6,done_lp
	moveq	#2,d4			; signal cave won
	move.l	points(a5),oscore(a5)	; current score= permanent score
	bra	rt_int2

;*********************************************************************
;
; this scan is called just to animate the sprites (and change amoebas)
;
;*********************************************************************

ani_rts rts

drwinbx addq.b	#1,ntryfrq(a5)
	move.b	ntryfrq(a5),d0
	subq.b	#7,d0
	bne.s	db_out
	move.b	d0,ntryfrq(a5)
	move.l	#snd_mwlr,d1
	bsr	sound
	bchg	#1,ntrstat(a5)
	movem.l d4/d5,-(a7)
	moveq	#0,d4
	move.l	entrylp(a5),d5		; animate live entry box
	moveq	#inbox,d0
	add.b	ntrstat(a5),d0		; this is 2 or 4, sprite for steelwall or exit box
	bsr	drw_spr
	movem.l (a7)+,d4/d5
	rts

drwbox	movem.l d4/d5,-(a7)
	moveq	#0,d4
	move.l	d0,d5
	moveq	#inbox,d0
	add.b	exitbit(a5),d0		; this is 2 or 4, sprite for steelwall or exit box
	bsr	drw_spr
	movem.l (a7)+,d4/d5
db_out	rts


; animate sprites
animate btst	#0,d0			; animate sprites? (only animate every second tick around)
	bne.s	ani_rts 		; ... no
	tst.w	mw_on(a5)		; is magic wall on?
	ble.s	no_cdwn 		; ... no
	bne.s	mw_cnt			; ... yes, and still counting down
	st	mw_on(a5)		; it's no longer on and can't be switched on
mw_cnt	move.w	mw_stat(a5),d0
	addq.w	#1,d0			; next magic wall animation status
	andi.w	#3,d0			; in range of 0...3
	move.w	d0,mw_stat(a5)

no_cdwn move.w	spr_ani(a5),d4		; current status of most animated sprites
	addq.w	#1,d4			; new status
	andi.w	#7,d4			; in range of 0 ... 7
	move.w	d4,spr_ani(a5)		; set new status
	mulu	spr_inc(a5),d4		; mulu on Q68 is faster than lsl
	move.l	array(a5),a4		; point to sbasic array

	move.w	amo_chg(a5),d6		; amoeba change status
	beq.s	anicont
	clr.w	amo_chg(a5)
	bra.s	anicont 		; jump over the following
					; (inserted here for short branches in loop)
; animate diggeridoo
digger	tst.w	d4			; same animation sequence ?
	bne.s	dgg_drw 		; ... yes
	cmp.w	#rfdboth,rok_ani(a5)	; ... no, so change it
	bgt.s	dgg_drw 		; but don't change if movement
	move.l	#$270025,d2		; get random nbr between 37 and 39
	bsr	rnd			; for new animation sequence
	move.w	d1,rok_ani(a5)		; diggeridoo sprite
	moveq	#0,d0
dgg_drw move.w	rok_ani(a5),d0		; sprite to draw
	bsr	drw_spr
	bra.s	ani_lp1

; treat special space : spaces briefly turn white when required nbr of diams reached
ani_spc tst.b	white(a5)		; special space drawing?
	beq.s	ani_lp1 		; ... no
	pea	ani_lp1
	blt	drw_blk 		; ... yes; redraw black, or
	bra	drw_wbl 		; ... yes, redraw white
	  
; now scan every square :
; here :

; d5 = (set below) row/col counter
; d6 = amoeba change state
; d7 = nbr of ticks between full scans
; a0 = screen in variable area
; a3 = timer
; a4 = start of row 2 of array (absolute)
; a5 = workspace
; a6 = basic ptr
anicont subq.l	#8,a4			; see ani_add below
	moveq	#nbrrows-2,d5		; there are 20 rows to inspect
ani_row
	swap	d5
	move.w	#nbrcols-2,d5		; there are 38 cols to check per row
ani_add addq.w	#8,a4			; jump over 0th & 1st elmnt of this row
					; and last 2 elmnts at end of previous row
ani_col moveq	#0,d0
	move.w	(a4)+,d0		; get sprite type
	beq.s	digger			; it's diggeridoo, animate it
	bgt.s	ani_plu
	neg.w	d0			; negative, reset status
	move.w	d0,-2(a4)		;
ani_plu cmp.w	#7,d0			; animated sprite?
	blt.s	ani_lp1 		; ... no, next square
	beq.s	ani_spc 		; ... no, handle space
	cmp.w	#explosion,d0		; explosion or higher?
	bge.s	ani_lp1 		; ... yes, not handled here
	cmp.w	#amoeba,d0		; amoeba?
	beq.s	amob			; ...yes, handle it
	cmp.w	#8,d0			; magic wall ?
	beq.s	mwall			; ... yes, handl it
redraw	bsr.s	drw_spr 		; draw sprite
ani_lp1 dbf	d5,ani_col		; treat all cols
	swap	d5
	dbf	d5,ani_row		; treat all rows
	tst.b	exit_ok(a5)		; exit is possible?
	beq.s	no_exit 		; ... no
	move.l	extloop(a5),d0		; ... yes, animate live exit box
	bsr	drwbox

no_exit tst.b	entry(a5)		; is diggeridoo still entering?
	beq.s	no_ntry 		; ... no
	bsr	drwinbx 		; yes, animate inbox

; check whether space needs to change colour
no_ntry tst.b	white(a5)
	beq.s	ani_out
	blt.s	ani_ex
	st	black(a5)
ani_ex	move.b	#0,white(a5)
ani_out rts

; amoeba drawing
amob	tst.w	d6			; amoebae need to change state?
	beq.s	redraw			; ... no
	blt.s	amo_dia 		; ... yes, convert to diamonds
	moveq	#boulder,d0		; ... or to boulders
amobset move.w	d0,-2(a4)
	bra.s	mw_draw 		; draw and continue loop
amo_dia moveq	#diamond,d0
	move.w	d0,-2(a4)
	bra.s	mw_draw
 
; animate magic wall
mwall	tst.w	mw_on(a5)		; is magic wall on ?
	ble.s	ani_lp1 		; ... no
	moveq	#mw_anim,d0
	add.w	mw_stat(a5),d0
mw_draw move.w	d4,-(a7)
	moveq	#0,d4
	bsr.s	drw_spr
	move.w	(a7)+,d4
	bra.s	ani_lp1

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;
; common subroutines
;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;



********************************************************
*
* draw a sprite when D5 is used as row/col counter
*
********************************************************

; d0 c	r   sprite nbr	(upper word : 0)  | 0
; d1-d3 s
; d4 c	 p  what part of an animation we're doing now
; d5 c	 p  row | col
; d6	 p
; a0 c	 p  screen address of variable area (after the steel walls)
; a1-a2  s
; a4 c	 p entry into array
; a5 c	 p mem ptr

drw_spr tst.w	d0
	bge.s	dw2
	neg.l	d0
dw2	add.w	d0,d0			; entries are word sized
	move.l	spr_tbl(a5),a1		; sprite list entries table
	move.w	(a1,d0.w),d0		; index into sprite list
	move.l	spr_gen(a5),a1		; point to sprites
	add.l	d0,a1			; this is the one we want
	add.w	d4,a1			; offset to current animation
drw_spr1
	bsr.s	calc_pos		; where does it go?
drw_spr2
	move.l	a0,a2			; screen address of my window (var area)
	add.l	d1,a2			; this is where we start drawing
	move.w	scr_spinc(a5),d2	; distance to move to next screen line
	move.w	scr_lop(a5),d3		; nbr of Lwords to move for 1 line of sprites
					; (already prepared for dbf)
	move.w	sp_ysiz(a5),d0		; sprite y size
	subq.w	#1,d0			; dbf
rowlp	move.w	d3,d1			; nb of LWs to move
collp	move.l	(a1)+,(a2)+
	dbf	d1,collp		; fill in 1 line of the sprite
	add.w	d2,a2			; advance to next screen line
	dbf	d0,rowlp		; do it for all lines
drw_out moveq	#0,d0
	rts

; calculate x,y position in pixels from row | col in squares
; (rel to wdw)
;
; !!!!!!!!
; this returns immediately to caller if column is outside drawable range
; ==-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=--=-=-=
; !!!!!!!!
;
; d0	s
; d1   r	x|y position in pixels
; d5 c	p	position as row | col (in the array) 0 = 1st usable position
;		in the array AFTER enclosing steelwall
;	       ---> NOTE: the position is given as DBF indexes!!!!
calc_pos
	move.w	d5,d0			; col given as...
	sub.w	#nbrcols-1,d0		; ... dbf index converted to ...
	neg.w	d0			; ... real col now
	move.w	startcol(a5),d1
	cmp.w	d1,d0			; before drawable start?
	blt.s	cp_ret			; ... yes, don't draw
	cmp.w	endcol(a5),d0		; beyond end of drawable range?
	bgt.s	cp_ret			; ... yes, don't draw
	sub.w	d1,d0			; in case we panned
	mulu	spr_lsz(a5),d0		; * x_size of one square in bytes
	move.l	d5,d1			; now the same for row with y size
	swap	d1
	sub.w	#nbrrows-1,d1
	neg.w	d1			; (don't swap d0, keep upper word 0)
	mulu	sp_ysiz(a5),d1		; nbr of lines per square/sprite
	mulu	scr_inc(a5),d1		; * nbr of bytes per line
	add.l	d0,d1
	rts
cp_ret	addq.l	#4,a7
	bra.s	drw_out

; draw a black block when D5 is used as row/col counter

; d0-d3 s
; d4 c	 p what part of an animation we're doing now
; d5 c	 p  row | col
; a0 c	 p  screen address of variable area (after the steel walls)
; a1-a2 s
; a4 c	 p entry into array
; a5 c	 p mem ptr

drw_blk2
	moveq	#0,d3			; draw black square w/o setting pos first
	bra.s	drw_bl3

drw_wbl moveq	#-1,d3			; draw white square
	bra.s	db_pos

drw_blk moveq	#0,d3			; draw black square, setting pos first

db_pos	bsr.s	calc_pos		; set screen position from row/col
drw_bl3 move.l	a0,a2			; screen address of my window
	add.l	d1,a2			; this is where we start drawing
	move.w	scr_spinc(a5),d2	; distance to move to next screen line
	move.w	sp_ysiz(a5),d0		; sprite y size
	subq.w	#1,d0
row_lp	move.w	scr_lop(a5),d1		; nb of LWs to move
col_lp	move.l	d3,(a2)+
	dbf	d1,col_lp		; fill in 1 line of the sprite
	add.w	d2,a2			; advance to next screen line
	dbf	d0,row_lp		; do it for all lines
	moveq	#0,D0
	rts


	end
