****************************************************
*
; BouQLder QLash
; scan loop action routine : check for diggeridoo move
; (C) W. Lenerz 2018
*
*****************************************************

	section basic

	include 'dev1_basic_boulder_asm_keys'
	include 'dev1_basic_boulder_asm_snd_keys'
	include 'dev8_keys_qdos_io'

	xdef	mv_diggr

	xref	drw_blk2
	xref	drw_spr2
	xref	nw_spc2
	xref	inc_dia
	xref	sound
	xref	chk_pan


; d0   r   0 if OK,  -1 if cave is finished by entering the exit box
; d1 c	s  key(s) hit
; d2 c	s  the object type
; d4 c	p  sprite animation counter
; d5 c	p  loop counter (row|col)
; d7 c	p  nbr of ticks betwen full scans
; a0 c	p  channel ID
; a3 c	p  timer address
; a4 c	p  ptr to NEXT element in array
; a5 c	p  workspace
; a6 c	p  basic ptr

; all other regs are volatile and may be used at will

****  add 1 in 8 chance for succeeding at pushing

cave_done
	moveq	#-1,d0
	rts

out	cmp.w	#rfdboth,rok_ani(a5)	; no move, but was move sprite shawn?
	ble.s	nochg			; ... no, so don't disturb anim sequence
	move.w	#rfdblnk,rok_ani(a5)	; ... yes, start blinking
nochg	moveq	#0,d0
exit	rts
mv_diggr
	andi.w	#214,d1 		; any of space or cursor keys pressed?
	beq.s	out			; ... no
	cmp.w	#64,d1
	beq.s	out			; space alone won't do
	move.l	rok_pos(a5),a2
	move.l	a2,a1			; same
	bclr	#6,d1			; space pressed?
	bne	space_k 		; ... yes, process space + cursor key
	moveq	#-1,d3
	btst	#1,d1			; 2   left
	bne.s	k_left
	btst	#2,d1			; 4   up
	bne.s	k_up
	btst	#4,d1			; 16  right
	bne.s	k_right
k_down	add.w	#rowsize,a2		; neither of above, so down, point row below, same col
	bra.s	k_comn
k_left	subq.l	#2,a2			; col to the left, same row
	moveq	#rfdleft,d3
	bra.s	k_comn
k_up	sub.w	#rowsize,a2		; row above, same col
	bra.s	k_comn
k_right addq.w	#2,a2
	moveq	#rfdrite,d3
; a2 is destination square, a1 is where I came from
k_comn	tst.w	d3
	blt.s	k_cmn2
	move.w	d3,rok_ani(a5)		; set sprite corresponding to movement
k_cmn2	move.w	(a2),d1 		; what is in destination?
	move.w	d1,d3
	subq.w	#3,d1			; is it dirt?
	beq.s	do_move 		; ... yes, so move there
	subq.w	#4,d1			; is it space?
	beq.s	do_move 		; ... yes, so move there
	  
	cmp.w	#aniexit-7,d1		; is it valid exit?
	beq.s	cave_done		; yes, cave finished

	subq.w	#2,d1			; is it diamond?
	bne.s	nochg			; ... no, so I can't go there
	bsr	inc_dia 		; inc dia value, make snd, set status bar
	moveq	#0,d3
; here I can move to the new position at (a2)
; a2 = new position (destination)
; a1 = old psition
do_move move.w	#0,(a2) 		; this is now diggeridoo
	tst.w	d3			; make sound?
	beq.s	no_snd			; ... no
	bsr	mk_snd

; now draw provenance square as empty space, and diggeridoo in destination
no_snd	move.l	a2,rok_pos(a5)		; new position for diggeridoo
	move.l	a1,d1
	movem.l d5/a2/a4,-(a7)
	move.l	a1,a4			; old
	addq.l	#2,a4			; point to next element
	bsr.s	calc_pos
	move.l	d1,d5
	bsr	drw_blk2		; draw old position as empty square
					; don't check for falling/rolling yet
	move.l	4(a7),d1		; d1 = new  position
	bsr	chk_pan 		; pan wdw if necessary
						       
drw_rfd bsr.s	calc_pos		; make into screen offset from (a0)
	move.l	spr_tbl(a5),a1		; sprite list entries table
	move.w	(a1),d0 		; index into sprite list
	move.l	spr_gen(a5),a1		; point to sprites
	add.l	d0,a1			; this is the one we want
	bsr	drw_spr2		; draw it at (a0,d1)
	bsr	nw_spc2 		; set space, check for falling/rolling object
	movem.l (a7)+,d5/a2/a4
k_out	moveq	#0,d0
	rts

; calculate screen position (rel to A0) from entry in array
; d1 entry in array
calc_pos
	sub.l	array(a5),d1		; relative to start of array
	divu	#rowsize,d1		; cols | rows
	add.l	#$20001,d1
	move.l	d1,d0
	mulu	sp_xsiz(a5),d1		; nbr of lines per square/sprite
	mulu	scr_inc(a5),d1		; * nbr of bytes per line
	swap	d0			; col
	lsr.w	#1,d0			; was word sized
	sub.w	startcol(a5),d0
	mulu	spr_lsz(a5),d0		; * x_size of one square in bytes
	add.l	d0,d1			; =screen pos of this array element
	rts

; space hit in conjunction with cursor keys : grab  / push
; a1 = a2 = current diggeridoo position
space_k moveq	#0,d0
	cmp.w	#2,d1			; 2   left
	beq.s	sk_left
	cmp.w	#4,d1			; 4   up
	beq.s	sk_up
	cmp.w	#16,d1			; 16  right
	beq.s	sk_rit
sk_down add.w	#rowsize,a2		; down, point row below, same col
	moveq	#-1,d0			; can't push boulder up/down
	bra.s	sk_comn
sk_left subq.w	#2,a2			; col to the left, same row
	bra.s	sk_comn
sk_up	sub.w	#rowsize,a2		; row above, same col
	moveq	#-1,d0			; can't push boulder
	bra.s	sk_comn
sk_rit	addq.w	#2,a2
; as of here, a2 is "destination", square to grab/push
sk_comn move.w	(a2),d2 		; what is in destination?
	subq.w	#1,d2			; is it a boulder?
	beq.s	psh_bldr		; ... yes, try to push it
	subq.w	#2,d2			; is it dirt?
	beq.s	sk_repl 		; ... yes, can do : replace with space
	subq.w	#6,d2			; is it diamond?
	beq.s	is_dia			; ... yes, grab it
	sub.w	#falldia-diamond,d2	; is it falling diamond?
	bne.s	sk_out			; ... no, so I can't do anything
is_dia	bsr	inc_dia 		; ... yes, grab it
sk_repl movem.l a4/d5,-(a7)
sk_rpl2 lea	2(a2),a4		; point to next element
	move.l	a2,d1
	bsr.s	calc_pos
	move.l	d1,d5
	bsr	drw_blk2		; draw space
	bsr	nw_spc2 		; set element, check for boulders/diams
	movem.l (a7)+,a4/d5
sk_out	moveq	#0,d0
	rts

; trying to push a boulder, d1 = direction, (a2)= boulder
psh_bldr
	tst.w	d0			; is push allowed (l/r only) ?
	bne.s	sk_out			; ... no
	move.l	a2,a1			; keep
	subq.l	#2,a1			; point to elent to the left of boulder
	subq.w	#2,d1			; was push left?
	beq.s	psh_try 		; ... yes
	addq.l	#4,a1			; point to element right
psh_try
	cmp.w	#7,(a1) 		; is push destination empty?
	bne.s	sk_out			; ... no, so I can't push
	moveq	#1,d0			; preset normal boulder
	cmp.w	#7,rowsize(a1)		; is empty space underneath destination?
	bne.s	sk_lp1			; ... no
	moveq	#-fallbdr,d0		; ... yes, set falling boulder
sk_lp1	move.w	d0,(a1) 		; this becomes boulder/falling boulder
	movem.l a4/d5,-(a7)
	move.l	a2,-(a7)
	move.l	a1,d1
	bsr	calc_pos		; make into screen offset from (a0)
	move.l	spr_tbl(a5),a1		; sprite list entries table
	moveq	#boulder+boulder,d0	; we want to draw a boulder
	move.w	(a1,d0.w),d0		; index into sprite list
	move.l	spr_gen(a5),a1		; point to sprites
	add.l	d0,a1			; this is the one we want
	bsr	drw_spr2		; draw it at (a0,d1)
	move.l	(a7)+,a2		; draw space where boulder used to be
	move.l	#snd_bdpu,d1
	bsr	sound
	bra.s	sk_rpl2

; make sound
mk_snd	moveq	#0,d1			; sound for empty space
	subq.w	#dirt,d3		; was it dirt?
	bne	sound			; ... no, good
	move.w	#snd_sand,d1		; ... yes, use this
	bra	sound

	end
